function [Elbow_set,Elbows,wn,w]=SICmethod_v4(V,paramFun)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Spectral_information_criterion                                     %%%%
%%% Luca Martino, Roberto San Millán, Eduardo Morgado                  %%%% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% Example:
%%%% k=0:1:50;
%%%% V=exp(-0.1*k.^2);
%%%% [Elbow_set,Elbows]=SICmethod_v4(V);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if nargin>=2
    N=paramFun.N;
    ell=paramFun.ell;
    plotyes=paramFun.plotyes;
else
    N=100000;
    ell=NaN;
    plotyes=1;
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%% with respect to the paper kt=k+1; 
kt=1:length(V);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
aux_amount=(V(1)-V(2:end))./[1:length(V)-1];
eta_max=max(aux_amount);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%
for jj=1:N  
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
     eta=0+eta_max*rand(1,1);
 %%%%%%%%%%%%%%%%%%%%%%  
   C=V+eta*(kt-1);
 %%%%%%%%%%%%%%%%%%%%%%%
    posaux=find(C==min(C));
    pos(jj)=posaux(1)-1;
%%%%%%%%%%%%%   
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%% compute PMF %%%
for i=1:max(kt)
    pos2=find(pos==(i-1));
    w(i)=length(pos2);
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
wn=w./sum(w);
%%%%%%%%%%%%%%%%%%%%%%%%
Elbow_set=unique(pos);
%%%%% the line above can be also done as:
%%%%% pos_elbowset=find(wn>0);
%%%%% Elbow_set=par(pos_elbowset)-1;
%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%
cumFun=cumsum(wn);
pos_elb90=find(cumFun>0.9);
Elbows(1)=kt(pos_elb90(1))-1;
pos_elb95=find(cumFun>0.95);
Elbows(2)=kt(pos_elb95(1))-1;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% WRITE Results                        %%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
disp('-----------------------------------------------------------------------------')
disp('  ')
disp('Results of the Spectral Information Criterion (SIC)')
disp('  ')
disp('-----------------------------------------------------------------------------')
disp('  ')
disp('Set of elbows (i.e., possible models with different number of components):')
disp(Elbow_set)
disp('-----------------------------------------------------------------------------')
disp('  ')
disp(['Number of possible models following SIC: ',num2str(length(Elbow_set))])
disp('  ')
disp('-----------------------------------------------------------------------------')
if isnan(ell)==0
    pos_elb_ell=find(cumFun>ell);
    Elbows(3)=kt(pos_elb_ell(1))-1;
    disp(['Suggested Elbows, 90% and 95% and ',num2str(ell),'% :'])
    disp(Elbows)
else
    disp('Final Suggested Elbows, 90% and 95% :')
    disp(Elbows)
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% JUST PLOTS               %%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if plotyes==1
        subplot(2,1,1)
        plot(kt-1,V,'bo-','LineWidth',2,'MarkerSize',7)
        set(gca,'Fontsize',25)
        axis([0 length(V)+1 min(V)-0.01 max(V)])
         xlabel('k')
        subplot(2,1,2)
        stem(kt-1,wn)
        axis([0 length(V)+1 0 1])
        set(gca,'Fontsize',25)
         xlabel('k')
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        figure
        stem(kt-1,wn,'LineWidth',2)
        axis([-2 length(V)+2 0 1])
         set(gca,'Fontsize',25)
         xlabel('k')
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        figure
        plot(kt-1,cumsum(wn),'bo-','LineWidth',2,'MarkerSize',10)
         set(gca,'Fontsize',25)
         xlabel('k')
         axis([-2 length(V)+2 0 1])
         hold on
         plot([-1 length(V)+1],[0.90 0.90],'k--','LineWidth',2)
         plot([-1 length(V)+1],[0.95 0.95],'k--','LineWidth',2)
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
